# This is the third script to run in the workflow. This script is used to print 
# out the analysis results on the parameters of the models run in the chain 
# runner script (e.g., model name, ML estimate of center and width, confidence 
# interval of center and width, loglikelihood and AIC).
source("JYHZARchainsR.R")
if(! any(file.exists(paste("cline-models/",c(traits,frogLoci),"MLE.RData",sep="") ) ))
  stop("No MLE process has been run for any trait. Please run JYHZARadmixAnalysis.R to generate model MLEs.")
if(length(frogcline.getSelectionSet())<1){

  stop("No MCMC process has been run for any trait. Please run JYChainRunner.R to generate tracecs.")
}
print(ll.cut <- runFrogAnalysis())

traits<-c("DorsalPhenotype",  "VentralPhenotype", "dorsal_lda_func",
          "admix.p.A","vent.Comp.1",  "dors.Comp.1")
traitsFreq <- c( "dorsal_lda_cat")
frogcline.getModels<-function(id){
  load(file=paste("cline-models/",id,"MLE.RData",sep=""))
  list(obsData=obsData,models=frogModels,init=frogInit,MLEs=frogMLE,AIC=frogAIC)
}
# tmp<- foreach(id=c(traits,traitsFreq),.combine=c) %do% { junk<-list() ; junk[[id]]<-frogcline.getModels(id)$AIC; junk }
# all.selected<-sapply(tmp, function(x) names(which(x<min(x+2))),simplify=FALSE)
all.selected<-frogcline.getSelectionSet()


mle.selected<- foreach(id=c(traits,traitsFreq),.combine=c) %do% { 
  junk<-list() ; junk[[id]]<-frogcline.getModels(id)$MLEs[all.selected[[id]]]; junk }

aic.selected<- foreach(id=c(traits,traitsFreq),.combine=c) %do% { 
  junk<-list() ; junk[[id]]<-frogcline.getModels(id)$AIC[all.selected[[id]]]; junk }

mle.cw <- foreach(id=names(mle.selected), .combine = rbind) %do% { 
  foreach(mod.id=names(mle.selected[[id]]), .combine = rbind) %do% { 
    cbind(trait=id,
          model=mod.id,
          as.data.frame(mle.selected[[id]][[mod.id]]$param.all)[,c("center","width")],
          logLik=mle.selected[[id]][[mod.id]]$logLik,
          AIC=frogcline.getModels(id)$AIC[[mod.id]]); 
  }
  }
print(mle.cw)

all.param <- unique(foreach(id=names(mle.selected), .combine = c) %do% { 
  foreach(mod.id=names(mle.selected[[id]]), .combine = c) %do% { 
    names(mle.selected[[id]][[mod.id]]$param.all) ; 
  }
})

mle.param <- foreach(id=names(mle.selected), .combine = rbind) %do% { 
  foreach(mod.id=names(mle.selected[[id]]), .combine = rbind) %do% {
    cbind(trait=id,
          model=mod.id,
          foreach(col.id=all.param, .combine = cbind) %do% { 
    if(col.id %in% names(mle.selected[[id]][[mod.id]]$param.all) ){
      as.data.frame(mle.selected[[id]][[mod.id]]$param.all)[,col.id,drop=F]
    } else {
      tmp<-data.frame(xxx=NA_real_) 
      names(tmp) <- col.id
      tmp
    }
  })
}
}

# print(mle.param)

if(F)
  {
model.subset<- c("free.none","fixed.both","free.both")
mle.alternative<- foreach(id=c(traits,traitsFreq),.combine=c) %do% { 
  junk<-list() ; 
  junk[[id]]<-frogcline.getModels(id)$MLEs[model.subset[model.subset %in% names(frogcline.getModels(id)$MLEs)]]; junk }

mle.cw_alt <- foreach(id=names(mle.alternative), .combine = rbind) %do% { 
  foreach(mod.id=names(mle.alternative[[id]]), .combine = rbind) %do% { 
    cbind(trait=id,
          model=mod.id,
          as.data.frame(mle.alternative[[id]][[mod.id]]$param.all)[,c("center","width")],
          logLik=mle.alternative[[id]][[mod.id]]$logLik,
          AIC=frogcline.getModels(id)$AIC[[mod.id]]); 
  }
}
print(mle.cw_alt)

}


if(F)
{
  frogcline.getModels_bkp<-function(id,key.i="0"){
    load(file=paste("cline-models/",id,"MLE_bkp",key.i,".RData",sep=""))
    list(obsData=obsData,models=frogModels,init=frogInit,MLEs=frogMLE,AIC=frogAIC)
  }
  str(frogcline.getModels_bkp("VentralPhenotype"), max.level = 1)
}


if(F)
{
  # Hunt for the official cline models
  clines.pub <- c("DorsalPhenotype", "VentralPhenotype", "admix.p.A", "vent.Comp.1", "dors.Comp.1")
  
  frogcline.getModels_archive<-function(id, 
                                        mle_dir="cline-models",
                                        mle_file=file.path(mle_dir,paste0(id,"MLE.RData")) ){
    load(file=mle_file)
    list(obsData=obsData,models=frogModels,init=frogInit,MLEs=frogMLE,AIC=frogAIC)
  }
  frogcline.loadAnalysis_archive<-function(id, 
                                           mle_dir="cline-analysis",
                                           mle_file=file.path(mle_dir,paste0(id,"model-data.RData")) ){
    load(file=mle_file)
    analysis.oDG
  }
  version_scanner <- function(mod.id,use.model){
    res <-list()
  frogcline.getModels_archive(mod.id)$MLEs[[use.model]]-> res$local
  try({
  frogcline.getModels_archive(mod.id,mle_dir="cline-models.pub")$MLEs[[use.model]] ->
    res$pub
  })
  frogcline.getModels_archive(mod.id,
                              mle_dir="/Users/asterion/Downloads/AmNat yeager 60080/HZAR files/HZAR files to run copy/AmNat60080_hzar_scripts/cline-models")$MLEs[[use.model]]-> 
    res$v20210712
  frogcline.getModels_archive(mod.id,
                              mle_dir="/Users/asterion/Downloads/AmNat yeager 60080/HZAR files/HZAR files to run copy/AmNat60080_hzar_scripts/cline-models copy")$MLEs[[use.model]] -> 
    res$v20210517
  frogcline.getModels_archive(mod.id,
                              mle_dir="/Users/asterion/Downloads/AmNat yeager 60080/HZAR files/HZAR files to run/cline-models")$MLEs[[use.model]] -> 
    res$v20150603
  try({
  frogcline.loadAnalysis_archive(mod.id,mle_dir="cline-analysis.pub")$data.groups[[use.model]]$ML.cline ->
    res$pub.mcmc; })
  try({
  frogcline.loadAnalysis_archive(mod.id,
                              mle_dir="/Users/asterion/Downloads/AmNat yeager 60080/HZAR files/HZAR files to run copy/AmNat60080_hzar_scripts/cline-analysis v2021")$data.groups[[use.model]]$ML.cline-> 
    res$v20210712mcmc ; })
  frogcline.loadAnalysis_archive(mod.id,
                              mle_dir="/Users/asterion/Downloads/AmNat yeager 60080/HZAR files/HZAR files to run copy/AmNat60080_hzar_scripts/cline-analysis")$data.groups[[use.model]]$ML.cline-> 
    res$v20210517mcmc
  frogcline.loadAnalysis_archive(mod.id,
                              mle_dir="/Users/asterion/Downloads/AmNat yeager 60080/HZAR files/HZAR files to run/cline-analysis")$data.groups[[use.model]]$ML.cline -> 
    res$v20150603mcmc
  res
  }
  cline.versions <- list()
  # use.model<- "free.none"
  for(mod.id in clines.pub[1:4]){
    
    cline.versions[[mod.id]] <- version_scanner(mod.id, "free.none") }
  for(mod.id in clines.pub[5]){
    
    cline.versions[[mod.id]] <- version_scanner(mod.id, "fixed.both") }
  
  pub.clines <- list()
  cline.versions$DorsalPhenotype$pub.mcmc -> pub.clines$Dorsal.cline
  cline.versions$VentralPhenotype$pub.mcmc -> pub.clines$Ventral.cline
  cline.versions$admix.p.A$pub -> pub.clines$Admix.cline
  cline.versions$vent.Comp.1$pub -> pub.clines$ventComp1.cline
  cline.versions$dors.Comp.1$pub -> pub.clines$dorsComp1.cline
  
  all.param <- unique(foreach(id=names(pub.clines), .combine = c) %do% { 
      names(pub.clines[[id]]$param.all) ; 
  })
  
  mle.param.pub <- foreach(id=names(pub.clines), .combine = rbind) %do% { 
    # foreach(mod.id=names(mle.selected[[id]]), .combine = rbind) %do% {
      cbind(trait=id,
            # model=mod.id,
            foreach(col.id=all.param, .combine = cbind) %do% { 
              if(col.id %in% names(pub.clines[[id]]$param.all) ){
                as.data.frame(pub.clines[[id]]$param.all)[,col.id,drop=F]
              } else {
                tmp<-data.frame(xxx=NA_real_) 
                names(tmp) <- col.id
                tmp
              }
            })
    
  }
  
  print(mle.param.pub)
  
  names(clines.pub) <- c("Dorsal", "Ventral", "Genotypic", "Ventral PC1", "Dorsal PC1")
  tmp<- foreach(id=clines.pub,.combine=c) %do% { junk<-list() ; junk[[id]]<-frogcline.getModels(id)$AIC; junk }
  all.selected<-sapply(tmp, function(x) names(which(x<min(x+2))),simplify=FALSE)
  
  # frogcline.getModels("DorsalPhenotype")$MLEs[[all.selected[["DorsalPhenotype"]]]] -> Dorsal.cline
  # frogcline.getModels("VentralPhenotype")$MLEs[[all.selected[["VentralPhenotype"]]]] -> Ventral.cline
  # frogcline.getModels("admix.p.A")$MLEs[[all.selected[["admix.p.A"]]]] -> Admix.cline
  # frogcline.getModels("vent.Comp.1")$MLEs[[all.selected[["vent.Comp.1"]]]] -> ventComp1.cline
  # frogcline.getModels("dors.Comp.1")$MLEs[[all.selected[["dors.Comp.1"]]]] -> dorsComp1.cline
  # 
  frogcline.getModels("DorsalPhenotype")$MLEs$free.none-> Dorsal.cline
  frogcline.getModels("VentralPhenotype")$MLEs$free.none-> Ventral.cline
  frogcline.getModels("admix.p.A")$MLEs$free.none-> Admix.cline
  frogcline.getModels("vent.Comp.1")$MLEs$free.none -> ventComp1.cline
  frogcline.getModels("dors.Comp.1")$MLEs$fixed.both -> dorsComp1.cline
  
  show_params <- function(cline.mle)  {
    cbind(#trait=id,
          # model=mod.id,
      LL=cline.mle$logLike,
          foreach(col.id=all.param, .combine = cbind) %do% { 
            if(col.id %in% names(cline.mle$param.all) ){
              as.data.frame(cline.mle$param.all)[,col.id,drop=F]
            } else {
              tmp<-data.frame(xxx=NA_real_) 
              names(tmp) <- col.id
              tmp
            }
          })
    
  }
  
  #  Official cline in paper, JYHZARfigures.R, 202109
  Dorsal.cline$clineFunc_plot <- function (x) 4-(
    3.99999995176237 + (6.81930659558594e-06 - 3.99999995176237) * 
      (1/(1 + exp(-4 * (x - 7.52373505032674)/0.861001112062098))))
  
  #  Official cline in paper, JYHZARfigures.R, 202109
  Ventral.cline$clineFunc_plot <- function (x) 4-(
    3.14703147971086 + (7.1641909753914e-09 - 3.14703147971086) * 
      (1/(1 + exp(-4 * (x - 7.25648733551233)/0.454851905147166))))
  
  # Official cline in paper, HZARPlotting.R, 20150708
  Admix.cline$clineFunc_plot <- function (x) 
    (1/(1 + exp(-4 * (x - 9.56175534196704)/0.0790422333058079)))
  
  # Official cline in paper, HZARPlotting.R, 20150708
  ventComp1.cline$clineFunc_plot <- function (x) 
    (1/(1 + exp(-4 * (x - 5.0202504069515)/2.6108968796832)))
  
  # Official cline in paper, HZARPlotting.R, 20150708
  dorsComp1.cline$clineFunc_plot <- function (x) 
    ifelse(x < 6.83720999762125 - 7.20673009695936e-15, 
           (exp(4 * 7.20673009695936e-15/2.86656092848213 * 
                  (0.224812047339705 - 1))/(1 + exp(-(4 * 7.20673009695936e-15/2.86656092848213))) * 
              exp(4 * (x - 6.83720999762125)/2.86656092848213 * 0.224812047339705)), 
           ifelse(x > 6.83720999762125 + 7.0234562575923, 
                  1 - (exp(4 * 7.0234562575923/2.86656092848213 * 
                             (0.521850997337677 - 1))/(1 + exp(-(4 * 7.0234562575923/2.86656092848213))) * 
                         exp(-4 * (x - 6.83720999762125)/2.86656092848213 * 
                               0.521850997337677)), (1/(1 + exp(-4 * (x - 6.83720999762125)/2.86656092848213)))))
  
  
  
  
  
}
