# This script should be run first in the workflow. This script generates maximum 
# likelihood estimates for all models of all traits. There is an option to print 
# to the screen information about the maximum likelihood estimates. 

library(hzar)
library(subplex)
library(dplyr)
if(require(doParallel)){
  cat(sprintf("Found %i cores on this machine.\n\n",parallel::detectCores()))
  registerDoParallel(parallel::detectCores())
} else {
  
  registerDoSEQ();
}

if(!dir.exists("cline-models")) dir.create("cline-models")

#Load Data

PhenUpdate <- read.csv("AmNat60080_phenotype.csv", na.strings="na")
PhenKeep <- PhenUpdate %>% pull("IndNumContin")
Phen <- read.csv("HZARPhenotypeFile.csv", na.strings="na") %>%
  #filter(IndNumContin %in% PhenKeep)
  full_join(PhenUpdate %>% select(IndNumContin,Locality,Loc_as_L,DorsalPheno,LID,Dorsal_Cluster,Dist_clust_center ),
             by=c("IndNumContin","Locality"))



#genotypes <- read.csv("~/Desktop/Pop Gen paper/HZAR files/HZAR files to run/HZARGenotypeFile.csv", na.strings="na")
#SVLSex <- read.csv("~/Desktop/Pop Gen paper/HZAR files/HZAR files to run/HZARSVLSexFile.csv", na.strings="na")


pair.loc<-read.csv("locPairDistNew.csv")
# pair.loc$pair.distance <- hzar.map.distanceFromSite(latLongSites = hzar.map.latLongSites(siteIDs = pair.loc$Locality,
#                                                                                          site.lat = pair.loc$DDLat, 
#                                                                                          site.long = pair.loc$DDLong),
#                                                     site0 = "L21")
# write.csv(pair.loc, file="locPairDistNew.csv")
# plot(hzar.map.distanceFromSite(latLongSites = hzar.map.latLongSites(siteIDs = pair.loc$Locality,site.lat = pair.loc$DDLat, site.long = pair.loc$DDLong),site0 = "L21") , pair.loc$pair.distance)


locTrans <- sprintf("L%02d",1:30)
#locTrans <- sprintf("L%02d",)
names(locTrans)<-
  c("A", "B",  "C", "D", "E", "F", "G",  "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T",  "U", "V", "W", "X", "Y", "Z","AA","BB","CC","DD")
#c("A", "B",  "C", "D", "E", "F", "G",  "H", "I", "J", "K", "L", "M", "N", "O", "S", "T",  "U", "V", "W", "X", "Y", "Z","AA","BB","CC","DD")
Phen <- Phen[!as.character(Phen$Locality) %in% c("P","Q","R" ,"X","Y") , ]
Phen$Locality<- factor(locTrans[as.character(Phen$Locality)] )

admixQT<-read.csv("HZARAdmixQTdata.csv")
rownames(admixQT) <- admixQT$ID
rownames(Phen) <- Phen$IndNumContin

Phen$admix.p.A<- admixQT[as.character(Phen$IndNumContin),"p.A"]

dorsalDFA <- read.csv("LDA_predict_ancestral_category.csv")
rownames(dorsalDFA) <-  dorsalDFA$IndNumContin
Phen$dorsal_lda_cat<- dorsalDFA[as.character(Phen$IndNumContin),"dorsal_lda_anc"]
Phen$dorsal_lda_func<- dorsalDFA[as.character(Phen$IndNumContin),"LD1"]

temp.getMLE <- function(fitR, nFits=1e2,doPar=FALSE){
  mP <- fitR$modelParam
  
  cat("\nBegin MLE estimation for model with parameters:\n")
  cat(names(mP$init), sep = " ",fill = T)
  cat("\n")
  res <- list()
  try({
  resF <- subplex(mP$init,
                  function(theta) -fitR$llFunc(theta),
                  control=list(parscale=sapply(names(mP$init),
                                               function(x) (mP$upper[[x]]-mP$lower[[x]])/50)))
  res <- list(hzar.gen.cline(resF$par, fitR))
  })
  cat("Randomly adjusting starting parameters to improve MLE\n")
  try({
  if(doPar ){
    res <- c(res,times(nFits-1) %dopar% {
      try({
      tmp <- subplex(temp.safe.init(fitR),
                     function(theta) -fitR$llFunc(theta),
                     control=list(parscale=sapply(names(mP$init),
                                                  function(x) (mP$upper[[x]]-mP$lower[[x]])/50)))
      return(list(hzar.gen.cline(tmp$par, fitR)))
      }); list(); })
  } else {
    res <- c(res,times(nFits-1) %do% {
      try({
        
      tmp <- subplex(temp.safe.init(fitR),
                     function(theta) -fitR$llFunc(theta),
                     control=list(parscale=sapply(names(mP$init),
                                                  function(x) (mP$upper[[x]]-mP$lower[[x]])/50)))
      return(list(hzar.gen.cline(tmp$par, fitR)))
      }); list(); })
  } })
  print(length(res))
  res[[which.max(sapply(res,function(x) x$logLike))[1]]]
}


temp.random.init <-
  function(modelParam) {
    sapply(names(modelParam$init),
           function(p) {
             if(p %in% c("varR","varL") )
               return(modelParam$init[[p]])
             runif(1,modelParam$lower[[p]],modelParam$upper[[p]] )
           },simplify=FALSE)
  }
temp.safe.init <- function(fitR,init=temp.random.init(fitR$modelParam),nAttempts=10){
  mLL=-1e8
  try(mLL <- fitR$llFunc(init) )
  while(mLL <= -1e8 && nAttempts >0){
    nAttempts=nAttempts-1
    init <- temp.random.init(fitR$modelParam)
    try(mLL <- fitR$llFunc(init) )
  }
  init
}





frogcline.phenObsData <- function(trait){
  
  hzar.doNormalData1DRaw(hzar.mapSiteDist(pair.loc$Locality,pair.loc$pair.distance),
                         Phen$Locality,
                         Phen[[trait]])
  
  
}

frogcline.freqObsData <- function(trait="dorsal_lda_cat"){
  stopifnot(trait=="dorsal_lda_cat")
  loc.map<- hzar.mapSiteDist(pair.loc$Locality,pair.loc$pair.distance)
  PhenLDA <- Phen[,c("Locality",trait)] %>%
    group_by(Locality) %>% summarise(dorsal.p=mean(dorsal_lda_cat=="Dark",na.rm=T),
                                     dorsal.n=sum(!is.na(dorsal_lda_cat))) %>% filter(dorsal.n>0) %>%
    mutate(dist= loc.map[Locality])
  with(PhenLDA, 
       hzar.doMolecularData1DPops(distance = dist,
                                  pObs =  dorsal.p,
                                  nEff = dorsal.n,
                                  siteID =  Locality  ))
  
  
}

DEBUG =F

tmp.model.addMinVariance <- function (meta.model, minValue) {
  meta.model <- hzar:::model.addMinParam(meta.model, quote(varL), 
                                  "varL", minValue)
  # meta.model <- hzar:::model.addMinParam(meta.model, quote(varH),
  #                                 "varH", minValue)
  meta.model <- hzar:::model.addMinParam(meta.model, quote(varR), 
                                  "varR", minValue)
  return(meta.model)
}
tmp.model.addMinCenterVariance <- function (meta.model, minValue) {
  meta.model <- hzar:::model.addMinParam(meta.model, quote(varH),
                                         "varH", minValue)
  return(meta.model)
}
frogcline.loadPhenModels <- function(trait,obsData=frogcline.phenObsData(trait)) {
  frogModels<-list()
  ## Make a helper function
  frogcline.loadModel <- function(scaling,tails,
                                  id=paste(scaling,tails,sep=".")){
    frogModels[[id]] <<-
      hzar.makeCline1DNormal(obsData, tails)
    ## As there is no quick option for "fixed" scaling, and the
    ## sites "A" and "L" have a fair number of samples (> 20),
    ## fix the mean and variance of the left and right sides of
    ## the cline to values observed at sites "A" and "L".
    if (all(regexpr("fixed",scaling,ignore.case=TRUE) == 1 )){
      hzar.meta.fix(frogModels[[id]])$muL <<- TRUE
      hzar.meta.fix(frogModels[[id]])$muR <<- TRUE
      hzar.meta.fix(frogModels[[id]])$varL <<- TRUE
      hzar.meta.fix(frogModels[[id]])$varR <<- TRUE
    }
    ## Site P is the "left" side of the cline, so pull the
    ## initial values from there.
    hzar.meta.init(frogModels[[id]])$muL <<-
      mean(na.omit(Phen[[trait]][Phen$Locality %in% c("L21","L14")]))
      #obsData$frame["L21","mu"]
    hzar.meta.init(frogModels[[id]])$varL <<-
      var(na.omit(Phen[[trait]][Phen$Locality %in% c("L21","L14")]))
      #obsData$frame["L21","var"]
    ## Sites S, T is the "right" side of the cline, so pull the
    ## initial values from there.
    hzar.meta.init(frogModels[[id]])$muR <<-
      mean(na.omit(Phen[[trait]][Phen$Locality %in% c("L23","L15")]))
    hzar.meta.init(frogModels[[id]])$varR <<-
      var(na.omit(Phen[[trait]][Phen$Locality %in% c("L23","L15")]))
    if(hzar.meta.init(frogModels[[id]])$varL == 0)
      hzar.meta.init(frogModels[[id]])$varL <<- 0.01
    if(hzar.meta.init(frogModels[[id]])$varR == 0)
      hzar.meta.init(frogModels[[id]])$varR <<- 0.01
    # # Set reasonable bounds on random initial values for variance
    # var.upper=max(4*diff(range(Phen[[trait]],na.rm=T))^2,
    #               hzar.meta.init(frogModels[[id]])$varR,
    #               hzar.meta.init(frogModels[[id]])$varH,
    #               hzar.meta.init(frogModels[[id]])$varL)
    # hzar.meta.upper(frogModels[[id]])$varR <<- var.upper
    # hzar.meta.upper(frogModels[[id]])$varH <<- var.upper
    # hzar.meta.upper(frogModels[[id]])$varL <<- var.upper
      
    cat("+")
      
  }
  frogcline.loadModel("fixed","none")
  frogcline.loadModel("free","none")
  frogcline.loadModel("fixed","mirror")
  frogcline.loadModel("free","mirror")
  frogcline.loadModel("fixed","right")
  frogcline.loadModel("free","right")
  frogcline.loadModel("fixed","left")
  frogcline.loadModel("free","left")
  frogcline.loadModel("fixed","both")
  frogcline.loadModel("free","both")
  #print(frogModels)
  ## Modify all models to focus on the region where the observed
  ## data were collected.
  ## Observations were between 0 and ~60 km.
  frogModels <- sapply(frogModels,
                       hzar.model.addBoxReq,
                       -10 , 70,
                       simplify=FALSE)
  # # Block tiny variance edge condition
  # frogModels <- sapply(frogModels,
  #                      tmp.model.addMinVariance,
  #                      1e-6,
  #                      simplify=FALSE)
  # frogModels <- sapply(frogModels,
  #                      tmp.model.addMinCenterVariance,
  #                      0.1,
  #                      simplify=FALSE)
  # For debugging, just provide the base models... 
  # if(DEBUG) return(frogModels)
  cat("\nBegin compiling.\n")
  #This is where we're building the MCMC functions to the model
  ## Compile each of the models to prepare for fitting
  frogInit <- sapply(frogModels,
                     hzar.first.fitRequest.gC,
                     obsData=obsData,
                     verbose=FALSE,
                     simplify=FALSE)
  cat("\nCompiling finished.\n")
  
  save(obsData,frogModels,frogInit,file=paste("cline-models/",trait,"FITREQ.RData",sep=""))
  cat("\nBegin MLE estimation for all models.\n")
  if(F){
  frogMLE <- sapply(frogInit, temp.getMLE, simplify=FALSE)
  } else {
    #Stupid variant with extra care:
    frogInitNMs <- names(frogInit)
    frogMLE <- list()
    if(is.null(frogInitNMs))
      frogInitNMs <- seq_len(length.out = frogInit)
    for(m_iter in frogInitNMs ){
      try({
        frogMLE[[m_iter]] = temp.getMLE(frogInit[[m_iter]])
      })
    }
    
  }
  if(length(frogMLE) == length(frogInit)){
  cat("\nMLE estimation for all models complete.\n")
  } else {
    cat("\nMLE estimation for some models did not complete.\n")
    save(obsData,frogModels,frogInit,frogMLE,file=paste("cline-models/",trait,"partial_eval.RData",sep=""))
  }
  cat("\n")
  print(frogAIC<- sapply( frogMLE, hzar.AIC.hzar.cline))
  out_file<- paste("cline-models/",trait,"MLE.RData",sep="")
  if(file.exists(out_file)){
    cnt=0
    while(file.exists(bkp_file<- paste("cline-models/",trait,"MLE_bkp", cnt,".RData",sep="")))
      cnt = cnt +1
    warning(sprintf("Renaming existing data from %s to %s.", out_file, bkp_file))
    file.rename(out_file, bkp_file)
  }
  save(obsData,frogModels,frogInit,frogMLE,frogAIC,file=paste("cline-models/",trait,"MLE.RData",sep=""))
  frogAIC
}

frogcline.loadFreqModels <- function(trait,obsData=frogcline.freqObsData(trait)) {
  frogModels<-list()
  ## Make a helper function
  frogcline.loadModel <- function(scaling,tails,
                                  id=paste(scaling,tails,sep=".")){
    frogModels[[id]] <<-
      hzar.makeCline1DFreq(obsData,scaling, tails)
    
    cat("+")
    
  }
  frogcline.loadModel("fixed","none")
  frogcline.loadModel("free","none")
  frogcline.loadModel("fixed","mirror")
  frogcline.loadModel("free","mirror")
  frogcline.loadModel("fixed","right")
  frogcline.loadModel("free","right")
  frogcline.loadModel("fixed","left")
  frogcline.loadModel("free","left")
  frogcline.loadModel("fixed","both")
  frogcline.loadModel("free","both")
  #print(frogModels)
  ## Modify all models to focus on the region where the observed
  ## data were collected.
  ## Observations were between 0 and ~60 km.
  frogModels <- sapply(frogModels,
                       hzar.model.addBoxReq,
                       -10 , 70,
                       simplify=FALSE)
  # For debugging, just provide the base models... 
  # if(DEBUG) return(frogModels)
  cat("\nBegin compiling.\n")
  #This is where we're building the MCMC functions to the model
  ## Compile each of the models to prepare for fitting
  frogInit <- sapply(frogModels,
                     hzar.first.fitRequest.old.ML,
                     obsData=obsData,
                     verbose=FALSE,
                     simplify=FALSE)
  cat("\nCompiling finished.\n")
  
  save(obsData,frogModels,frogInit,file=paste("cline-models/",trait,"FITREQ.RData",sep=""))
  cat("\nBegin MLE estimation for all models.\n")
  if(F){
    frogMLE <- sapply(frogInit, temp.getMLE, simplify=FALSE)
  } else {
    #Stupid variant with extra care:
    frogInitNMs <- names(frogInit)
    frogMLE <- list()
    if(is.null(frogInitNMs))
      frogInitNMs <- seq_len(length.out = frogInit)
    for(m_iter in frogInitNMs ){
      try({
        frogMLE[[m_iter]] = temp.getMLE(frogInit[[m_iter]])
      })
    }
    
  }
  if(length(frogMLE) == length(frogInit)){
    cat("\nMLE estimation for all models complete.\n")
  } else {
    cat("\nMLE estimation for some models did not complete.\n")
    save(obsData,frogModels,frogInit,frogMLE,file=paste("cline-models/",trait,"partial_eval.RData",sep=""))
  }
  cat("\n")
  print(frogAIC<- sapply( frogMLE, hzar.AIC.hzar.cline))
  out_file<- paste("cline-models/",trait,"MLE.RData",sep="")
  if(file.exists(out_file)){
    cnt=0
    while(file.exists(bkp_file<- paste("cline-models/",trait,"MLE_bkp", cnt,".RData",sep="")))
      cnt = cnt +1
    warning(sprintf("Renaming existing data from %s to %s.", out_file, bkp_file))
    file.rename(out_file, bkp_file)
  }
  save(obsData,frogModels,frogInit,frogMLE,frogAIC,file=paste("cline-models/",trait,"MLE.RData",sep=""))
  frogAIC
}

#traits<-c("DorsalPhenotype" , "DorsR" ,"DorsG"  ,"DorsB","VentralPhenotype","VentR" ,"VentG", "VentB","admix.p.A")
#traits<-c("DorsalPhenotype",  "VentralPhenotype", 
 #         "admix.p.A", "vent.Comp.1",  "dors.Comp.1")

traits<-c("DorsalPhenotype",  "VentralPhenotype", "dorsal_lda_func",
          "admix.p.A","vent.Comp.1",  "dors.Comp.1")
traitsFreq <- c( "dorsal_lda_cat")

frogcline.loadPhenModels(traits[3])
frogcline.loadPhenModels(traits[2])
tmp<- foreach(id=traits[-c(2,3)]) %dopar% { frogcline.loadPhenModels(id); }
tmp2 <- frogcline.loadFreqModels(traitsFreq[1])
frogcline.getModels<-function(id){
  load(file=paste("cline-models/",id,"MLE.RData",sep=""))
  list(obsData=obsData,models=frogModels,init=frogInit,MLEs=frogMLE,AIC=frogAIC)
}
tmp<- foreach(id=c(traits,traitsFreq),.combine=c) %do% { junk<-list() ; junk[[id]]<-frogcline.getModels(id)$AIC; junk }
all.selected<-sapply(tmp, function(x) names(which(x<min(x+2))),simplify=FALSE)

mle.selected<- foreach(id=c(traits,traitsFreq),.combine=c) %do% { 
  junk<-list() ; junk[[id]]<-frogcline.getModels(id)$MLEs[all.selected[[id]]]; junk }

aic.selected<- foreach(id=c(traits,traitsFreq),.combine=c) %do% { 
  junk<-list() ; junk[[id]]<-frogcline.getModels(id)$AIC[all.selected[[id]]]; junk }

cat("\nAIC of top models:\n")
print(aic.selected)

if(T) # print center / width details about all models 
{
  model.subset<- 
    paste(rep(c("fixed","free"),each=5),
          c("none","left","right","mirror","both"), sep=".")
    
  mle.alternative<- foreach(id=c(traits,traitsFreq),.combine=c) %do% { 
    junk<-list() ; 
    junk[[id]]<-frogcline.getModels(id)$MLEs[model.subset[model.subset %in% names(frogcline.getModels(id)$MLEs)]]; junk }
  
  mle.cw_alt <- foreach(id=names(mle.alternative), .combine = rbind) %do% { 
    foreach(mod.id=names(mle.alternative[[id]]), .combine = rbind) %do% { 
      cbind(trait=id,
            model=mod.id,
            as.data.frame(mle.alternative[[id]][[mod.id]]$param.all)[,c("center","width")],
            logLik=mle.alternative[[id]][[mod.id]]$logLik,
            AIC=frogcline.getModels(id)$AIC[[mod.id]]); 
    }
  }
  print(mle.cw_alt)
  
  # print all parameters for all models ?
  if(F)
  {
    # mle.to.scan <- mle.selected
    mle.to.scan <- 
      mle.alternative[c("DorsalPhenotype", "VentralPhenotype", "admix.p.A", 
                        "vent.Comp.1",  "dors.Comp.1")]
    all.param <- unique(foreach(id=names(mle.to.scan), .combine = c) %do% { 
      foreach(mod.id=names(mle.to.scan[[id]]), .combine = c) %do% { 
        names(mle.to.scan[[id]][[mod.id]]$param.all) ; 
      }
    })
    
    mle.param <- foreach(id=names(mle.to.scan), .combine = rbind) %do% { 
      foreach(mod.id=names(mle.to.scan[[id]]), .combine = rbind) %do% {
        cbind(trait=id,
              model=mod.id,
              foreach(col.id=all.param, .combine = cbind) %do% { 
                if(col.id %in% names(mle.to.scan[[id]][[mod.id]]$param.all) ){
                  as.data.frame(mle.to.scan[[id]][[mod.id]]$param.all)[,col.id,drop=F]
                } else {
                  tmp<-data.frame(xxx=NA_real_) 
                  names(tmp) <- col.id
                  tmp
                }
              },
              logLik=mle.alternative[[id]][[mod.id]]$logLik,
              AIC=frogcline.getModels(id)$AIC[[mod.id]])
      }
    }
    
    print(mle.param)
  }
  
}

