# This is a utility script that is automatically loaded by the code files, 
# JYChainRunner.R and AmNat_20210820_extras.R


library(hzar)
library(subplex)
if(require(doParallel)){
  cat(sprintf("Found %i cores on this machine.\n\n",parallel::detectCores()))
  registerDoParallel(parallel::detectCores())
} else {
  
  registerDoSEQ();
}

frogLoci<- character(0)#c("X44","X92","X110","G5","C11","E3","F1","B8","B9","C3","D4","O1" )

traits<-c("dorsal_lda_func","dorsal_lda_cat" ,"DorsalPhenotype", "VentralPhenotype", "admix.p.A", "vent.Comp.1", "dors.Comp.1")

all.selected <- 
  list(DorsalPhenotype = "free.none",
       VentralPhenotype = "free.none",
       admix.p.A = "free.none",
       vent.Comp.1 = "free.none",
       dors.Comp.1 = "fixed.both",
       dorsal_lda_func = "free.none", 
       dorsal_lda_cat = c("free.none", "fixed.left")); 

frogcline.getModels<-function(locus){
  if(! any(file.exists(paste("cline-models/",locus,"MLE.RData",sep="") ) ))
    stop("No MLE process has been run for ",locus,". Please run JYHZARadmixAnalysis.R to generate model MLEs.")
  load(file=paste("cline-models/",locus,"MLE.RData",sep=""))
  list(obsData=obsData,models=frogModels,init=frogInit,MLEs=frogMLE,AIC=frogAIC)
}
if(!exists("all.selected") || is.null(all.selected)) {
  if(! any(file.exists(paste("cline-models/",c(traits,frogLoci),"MLE.RData",sep="") ) ))
    stop("No MLE process has been run for any trait. Please run JYHZARadmixAnalysis.R to generate model MLEs.")
tmp<- foreach(id=c(traits,frogLoci),.combine=c) %do% { junk<-list() ; junk[[id]]<-frogcline.getModels(id)$AIC; junk }
#all.selected<-sapply(tmp, function(x) names(which(x<min(x+2))),simplify=FALSE)
all.selected<-sapply(tmp, function(x) names(which(!is.na(x))),simplify=FALSE)
}

frogcline.getSelectionSet <- function(scan.traits=c(frogLoci.sel,traits.sel) ){
  foreach(id=scan.traits, .combine=c) %do% {
    if(!file.exists(paste("cline-analysis/",id,"model-data.RData",sep=""))) return(list())
    res<- local({
    load(file=paste("cline-analysis/",id,"model-data.RData",sep=""))
      list(names(analysis.oDG$data.groups))
    })
    names(res)<- id
    res
  }
}

frogcline.getTrace<- function(locus,model.selected=all.selected[[locus]],doPar=FALSE,tuneVarEdge=1,tuneCW=1){
  models=frogcline.getModels(locus)
  cat("A")
  obsData=models$obsData
  for( id in model.selected){
    if("varL" %in% names(models$init[[id]]$modelParam$tune))
    models$init[[id]]$modelParam$tune$varL   <- models$init[[id]]$modelParam$tune$varL * tuneVarEdge
    if("varR" %in% names(models$init[[id]]$modelParam$tune))
    models$init[[id]]$modelParam$tune$varR   <- models$init[[id]]$modelParam$tune$varR * tuneVarEdge
    models$init[[id]]$modelParam$tune$center   <- models$init[[id]]$modelParam$tune$center * tuneCW
    models$init[[id]]$modelParam$tune$width   <- models$init[[id]]$modelParam$tune$width * tuneCW
    
  }
  init=hzar.multiFitRequest(models$init[model.selected],
                            skip=which(locus==c(frogLoci,traits))*100);
  print(init)
  runInit=hzar.doFit.multi(init,doPar=doPar)
  save(locus,model.selected,obsData,init,runInit,file=paste("cline-models/",locus,"Init.RData",sep=""))
  cat("B")
  if(doPar){
    chainInit=mclapply(runInit,
                       hzar.next.fitRequest,mc.cores=3)
  } else {
    chainInit=lapply(runInit,
                     hzar.next.fitRequest) 
  }
  chainInit=hzar.multiFitRequest(chainInit,
                                 each=3,
                                 baseSeed=NULL)
  chainRuns=hzar.doChain.multi(chainInit,doPar=doPar)
  
  save(locus,model.selected,obsData,chainInit,chainRuns,file=paste("cline-models/",locus,"Chains.RData",sep=""))
  cat("C")
#   frogAnalysis <- hzar.data
#   
#   save(locus,model.selected,obsData,frogAnalysis,file=paste("cline-models/",locus,"Analysis.RData",sep=""))
#   cat("D\n")
}

frogcline.loadInit<- function(id) {
  local({
    load(file=paste("cline-models/",id,"Init.RData",sep=""))
    list(cline.name=locus, m.names=model.selected,obsData=obsData,
         init=init,
         runs=runInit)
  
  })
  #load(file=paste("cline-models/",id,"Chains.RData",sep=""))
}

frogcline.loadTrace<- function(id) {
  local({
    load(file=paste("cline-models/",id,"Init.RData",sep=""))
    init
  }) -> models
  load(file=paste("cline-models/",id,"Chains.RData",sep=""))
  list(cline.name=locus, m.names=model.selected,obsData=obsData,
       initDG=sapply(models,hzar.dataGroup.add,simplify=FALSE),
       runs=chainRuns)
}
#Start here and build analysis
#Run on everything from line 54- 72, then do hzar.getLLCutParam
#Type load("cline-analysis/C11model-data.RData")

frogcline.buildAnalysis <- function(id) {
  local({
    frogcline.loadTrace(id)-> trace
    trace_init<-
      sapply(trace$runs, 
             function(x) any(attr(x[[1]], "fit.success"),
                             attr(x[[2]], "fit.success"),
                             attr(x[[3]], "fit.success") ) )
    if(any(trace_init)) {
    if(!all(sapply(trace$runs, 
                   function(x) attr(x[[3]], "fit.success"))))
      warning("Some chains failed for ", id)
    chainDG<-lapply(trace$runs[trace_init], 
                    function(x) {
                      if(attr(x[[3]], "fit.success"))
                        return(hzar.dataGroup.add(x[[3]],doPar = TRUE));
                      if(attr(x[[2]], "fit.success"))
                        return(hzar.dataGroup.add(x[[2]],doPar = TRUE));
                      return(hzar.dataGroup.add(x[[1]],doPar = TRUE));
                      } )
    } else {
      chainDG<-lapply(trace$runs, 
                      function(x) hzar.dataGroup.add(x[[3]],doPar = TRUE) )
    }
    oDG<-hzar.make.obsDataGroup(chainDG)
    oDG<-hzar.copyModelLabels(trace$initDG,oDG)
    oDG
  })->analysis.oDG
  
  save(analysis.oDG,file=paste("cline-analysis/",id,"model-data.RData",sep=""))
}



frogLoci.sel<- character(0);#c("X44","X92","C11","E3","F1")

traits.sel<-c("dorsal_lda_func","dorsal_lda_cat" ,"DorsalPhenotype", "VentralPhenotype", "admix.p.A", "vent.Comp.1", "dors.Comp.1")
runFrogChainsAndAnalyze <- function(resREQ =names(all.selected)){
  # resREQ=c(frogLoci.sel,traits.sel)
  
tmp<-foreach(id=resREQ) %dopar% { frogcline.getTrace(id); NULL}
tmp<-foreach(id=resREQ) %dopar% { frogcline.buildAnalysis(id); NULL}

foreach(id= resREQ) %do% { load(file=paste("cline-analysis/",id,"model-data.RData",sep=""))
                                                hzar.getLLCutParam (analysis.oDG$data.groups,c("center", "width"))}
}

traits.analyzed<-c("dorsal_lda_func","dorsal_lda_cat" ,"DorsalPhenotype", "VentralPhenotype", "admix.p.A", "vent.Comp.1", "dors.Comp.1")

runFrogAnalysis <- function( resREQ =names(all.selected)){
  # resREQ=c(frogLoci.sel, traits.analyzed)
 
res<-foreach(id= resREQ) %do% { 
  load(file=paste("cline-analysis/",id,"model-data.RData",sep=""))
  hzar.getLLCutParam (analysis.oDG$data.groups,c("center", "width"))}
names(res)<-resREQ
res
}

