# This is a script that plots Figures 3 and S2. It requires the analysis script 
# files above to be run first.

library(hzar)

frogcline.loadAnalysis<-function(id) {
  load(file=paste("cline-analysis/",id,"model-data.RData",sep=""))
  analysis.oDG
}

gCenter<-function(dG) dG$ML.cline$param.all$center
gWidth<-function(dG) dG$ML.cline$param.all$width

# Dorsal and Ventral Scores
dScores.dG <- frogcline.loadAnalysis("DorsalPhenotype")$data.groups$free.none
vScores.dG <- frogcline.loadAnalysis("VentralPhenotype")$data.groups$free.none

#Remove defective data:
keep.v=foreach(ll=unique( vScores.dG$data.LL$model.LL),.combine=c) %do%
  which(vScores.dG$data.LL$model.LL==ll)[1]
keep.d=foreach(ll=unique(dScores.dG$data.LL$model.LL),.combine=c) %do%
  which(dScores.dG$data.LL$model.LL==ll)[1]

dScores.dG$data.mcmc  <- dScores.dG$data.mcmc[keep.d,,drop=FALSE]
dScores.dG$data.param <- dScores.dG$data.param[keep.d,,drop=FALSE]
dScores.dG$data.LL    <- dScores.dG$data.LL[keep.d,,drop=FALSE]

vScores.dG$data.mcmc  <- vScores.dG$data.mcmc[keep.v,,drop=FALSE]
vScores.dG$data.param <- vScores.dG$data.param[keep.v,,drop=FALSE]
vScores.dG$data.LL    <- vScores.dG$data.LL[keep.v,,drop=FALSE]
fg.getCredParamAll <-function (dataGroup,reject=0.05) {
  junk <- hzar.getCredParam(dataGroup,reject)
  junk1 <- hzar:::PP.fzCline.reduce(junk)
  junk1 <- hzar:::PP.fzCline.reduce(junk1)
  res <- hzar.make.fzCline(lapply(1:(dim(junk1)[[1]]), 
                                  function(x) hzar.gen.cline(junk1[x, ], dataGroup)))
  return(res)
}
vScores.fzCline <- fg.getCredParamAll(vScores.dG,1e-10)
dScores.fzCline <- fg.getCredParamAll(dScores.dG,0)

## flip the clines artificially
flipLF<-function (xVal, funcList = res$clines) {
  yList <- as.numeric(lapply(funcList, function(x, u) x$clineFunc(u),  u = xVal))
  return(data.frame(x = xVal, yMin = 4-max(yList), yMax = 4-min(yList)))
}
dScores.fzCline$listFuncInt <- flipLF
environment(dScores.fzCline$listFuncInt) <- environment(dScores.fzCline$fzCline)
assign("res",value = dScores.fzCline,envir = environment(dScores.fzCline$listFuncInt))
environment(dScores.fzCline$listFuncInt)
vScores.fzCline$listFuncInt <- flipLF
environment(vScores.fzCline$listFuncInt) <- environment(vScores.fzCline$fzCline)
assign("res",value = vScores.fzCline,envir = environment(vScores.fzCline$listFuncInt))
environment(vScores.fzCline$listFuncInt) 
print(vScores.dG$ML.cline$clineFunc)
# vScores.dG$ML.cline$clineFunc<-function (x) 4-(
#   3.14703147971086 + (7.1641909753914e-09 - 3.14703147971086) * 
#     (1/(1 + exp(-4 * (x - 7.25648733551233)/0.454851905147166))))
body(vScores.dG$ML.cline$clineFunc)<-
  eval(substitute(quote(4-tmp), list(tmp=body(vScores.dG$ML.cline$clineFunc))))
print(vScores.dG$ML.cline$clineFunc)

# print(dScores.dG$ML.cline$clineFunc)
# dScores.dG$ML.cline$clineFunc <- function (x) 4-(
#   3.99999995176237 + (6.81930659558594e-06 - 3.99999995176237) * 
#     (1/(1 + exp(-4 * (x - 7.52373505032674)/0.861001112062098))))
print(vScores.dG$ML.cline$clineFunc)
# vScores.dG$ML.cline$clineFunc<-function (x) 4-(
#   3.14703147971086 + (7.1641909753914e-09 - 3.14703147971086) * 
#     (1/(1 + exp(-4 * (x - 7.25648733551233)/0.454851905147166))))
body(dScores.dG$ML.cline$clineFunc)<-
  eval(substitute(quote(4-tmp), list(tmp=body(dScores.dG$ML.cline$clineFunc))))
print(dScores.dG$ML.cline$clineFunc)

#Descending complement of Q.A2
QB2.fzCline <- hzar.getCredParamRed(QB2.dG <- frogcline.loadAnalysis("admix.p.A")$data.groups$free.none)



if(! exists("dors_func.dG") ){
  dors_func.dG <- frogcline.loadAnalysis("dorsal_lda_func")$data.groups$free.none
  dors_func.fzCline <- hzar.getCredParamRed( dors_func.dG)
  if(T){ # flip y axis from [-6, 3] to [3, -6]
    dfa_flipLF<-function (xVal, funcList = res$clines) {
      yList <- as.numeric(lapply(funcList, function(x, u) x$clineFunc(u),  u = xVal))
      return(data.frame(x = xVal, yMin = -3-max(yList), yMax = -3-min(yList)))
    }
    
    dors_func.fzCline$listFuncInt <- dfa_flipLF
    environment(dors_func.fzCline$listFuncInt) <- environment(dors_func.fzCline$fzCline)
    assign("res",value = dors_func.fzCline,envir = environment(dors_func.fzCline$listFuncInt))
    
    # dors_func.dG$ML.cline$clineFunc <- function (x) 
    #   (-3-(0.916811002547307 + (-5.46224735467246 - 0.916811002547307) * 
    #   (1/(1 + exp(-4 * (x - 7.47343091688871)/2.81347932708251)))))
    body(dors_func.dG$ML.cline$clineFunc)<-
      eval(substitute(quote(-3-tmp), list(tmp=body(dors_func.dG$ML.cline$clineFunc))))
  }
}
if(! exists("dors_cat.dG") ){
  dors_cat.dG <- frogcline.loadAnalysis("dorsal_lda_cat")$data.groups$fixed.left
  dors_cat.fzCline <- hzar.getCredParamRed( dors_cat.dG)
  # Alternative to add free.none model results
  if(F){
    dors_cat.dG_extra <-  frogcline.loadAnalysis("dorsal_lda_cat")$data.groups$free.none
    dors_cat.fzCline$clines=c(dors_cat.fzCline$clines,
                              hzar.getCredParamRed( dors_cat.dG_extra)$clines)
    environment(dors_cat.fzCline$listFuncInt)
    assign("res",value = dors_cat.fzCline,
           envir = environment(dors_cat.fzCline$listFuncInt))
  }
}
qt.key<-c("dors", "QA4", "QB2", "QB4", "QD4", "vent","dScores","vScores","dors_func","dors_cat")
qt.info <- 
  data.frame(qt.name=c("Dorsal PC1",qt.key[2:5],"Ventral PC1","Dorsal Score","Ventral Score",
                       "Dorsal Discriminant Function","Dorsal DFA Category"),
             col=c("grey10","blue","grey20","orange","red","grey50","grey10","grey50",
                   "grey50","grey50"),
             dG.name=paste0(qt.key,".dG"),
             fz.name=paste0(qt.key,".fzCline") ,
             row.names = qt.key)
qt.col<-as.list(structure(as.character(qt.info$col),.Names=qt.key))

alpha.adjust=0.5




pdf(file="FrogCline_QvsTrait_plots032022_nobars.pdf",width=9.42,height = 8.74)
par(mfrow=c(2,1),
    mai=c(1.2,1.2,0,0.8), 
    oma=c(2,0,1,0)+0)

plot(0, pch = "",xlim = c(0,18), ylim = c(0,1), lwd = 0.5,
     xlab="",ylab="Genotypic Transition",yaxt='none',cex.lab=1.8)
hzar.plot.fzCline(dataGroup =  QB2.dG,fzCline = QB2.fzCline,
                  pch=1,lty=2,add=TRUE, col="black",#qt.col$QB2, 
                  fzCol = adjustcolor(qt.col$QB2, alpha = alpha.adjust),
                  xlim=c(0,18),ylim=c(0,1),ylab="Allele Frequency")
# hzar.plot.obsData()
# text(17.5,0.1,"K = 2")
# mtext(side=1,adj=1,text="Q2",line=0.8)
axis(2, at=seq(0,1,by=0.2))
# rect(gCenter(QB2.dG)-gWidth(QB2.dG)/2, 1.1,gCenter(QB2.dG)+gWidth(QB2.dG)/2, 1.15,
#      density = NULL, angle = 45, col = adjustcolor(qt.col$QB2, alpha = alpha.adjust),
#      border = adjustcolor(qt.col$QB2, alpha = alpha.adjust), lty = par("lty"), lwd = par("lwd"))
abline(v=gCenter(QB2.dG), col = adjustcolor(qt.col$QB2, alpha = alpha.adjust), lty="dashed", lwd=3 )

plot(0, pch = "",xlim = c(0,18), ylim = c(0,4), lwd = 0.5,
     xlab="Distance (km)",ylab="Phenotypic Transition",yaxt='none',cex.lab=1.8)
hzar.plot.fzCline(dataGroup =  dScores.dG,fzCline = dScores.fzCline,
                  pch="",lty=2,add=TRUE, col="black",#qt.col$dScores, 
                  fzCol = adjustcolor(qt.col$dScores, alpha = alpha.adjust),
                  xlim=c(0,18),ylim=c(0,1),ylab="Allele Frequency")
orig_obs_data <- hzar.extract.obsData(dScores.dG)$frame
points(y = 4-orig_obs_data$mu, x = orig_obs_data$dist, type = "p", pch = 4, col="black")
hzar.plot.fzCline(dataGroup =  vScores.dG,fzCline = vScores.fzCline,
                  pch="",lty=2,add=TRUE, col="black",#qt.col$vScores, 
                  fzCol = adjustcolor(qt.col$vScores, alpha = alpha.adjust),
                  xlim=c(0,18),ylim=c(0,1),ylab="Allele Frequency")
orig_obs_data <- hzar.extract.obsData(vScores.dG)$frame
points(y = 4-orig_obs_data$mu, x = orig_obs_data$dist, type = "p", pch = 3, col="black")
text(17.5,0.1,"Color Score")
axis(2, at=seq(0,4,by=1),labels = seq(4,0,-1))
# rect(gCenter(dScores.dG)-gWidth(dScores.dG)/2, 4.15,gCenter(dScores.dG)+gWidth(dScores.dG)/2, 4.55,
#      density = NULL, angle = 45, col = adjustcolor(qt.col$dScores, alpha = alpha.adjust),
#      border = adjustcolor(qt.col$dScores, alpha = alpha.adjust), lty = par("lty"), lwd = par("lwd"))

abline(v=gCenter(vScores.dG), col = adjustcolor(qt.col$vScores, alpha = alpha.adjust), 
       lty="dashed", lwd=3 )

# rect(gCenter(vScores.dG)-gWidth(vScores.dG)/2, 4.65,gCenter(vScores.dG)+gWidth(vScores.dG)/2, 5.05,
#      density = NULL, angle = 45, col = adjustcolor(qt.col$vScores, alpha = alpha.adjust),
#      border = adjustcolor(qt.col$vScores, alpha = alpha.adjust), lty = par("lty"), lwd = par("lwd"))

abline(v=gCenter(dScores.dG), col = adjustcolor(qt.col$dScores, alpha = alpha.adjust), 
       lty="dashed", lwd=3 )

dev.off()

pdf(file="FrogCline_QvsTrait_plots032022_dfa.pdf",width=9.42,height = 8.74)
par(mfrow=c(2,1),
    mai=c(1.2,1.2,0,0.8), 
    oma=c(2,0,1,0)+0)

plot(0, pch = "",xlim = c(0,18), ylim = c(0,1), lwd = 0.5,
     xlab="Distance (km)",ylab="Phenotypic Transition",yaxt='none',cex.lab=1.8)
hzar.plot.fzCline(dataGroup =  dors_cat.dG,fzCline = dors_cat.fzCline,
                  pch=4,lty=2,add=TRUE, col="black",#qt.col$QB2, 
                  fzCol = adjustcolor(qt.col$dors_cat, alpha = alpha.adjust),
                  xlim=c(0,18),ylim=c(0,1),ylab="Allele Frequency")
# hzar.plot.obsData()
text(12.5,0.1,"Categorical PC1")
# mtext(side=1,adj=1,text="Q2",line=0.8)
axis(2, at=seq(0,1,by=0.2))

abline(v=gCenter(dors_cat.dG), col = adjustcolor(qt.col$dors_cat, alpha = alpha.adjust), 
       lty="dashed", lwd=3 )

plot(0, pch = "",xlim = c(0,18), ylim = c(-6,3), lwd = 0.5,
     xlab="Distance (km)",ylab="Phenotypic Transition",yaxt='none',cex.lab=1.8)
hzar.plot.fzCline(dataGroup =  dors_func.dG,fzCline = dors_func.fzCline,
                  pch="",lty=2,add=TRUE, col="black",#qt.col$QB2, 
                  fzCol = adjustcolor(qt.col$dors_func, alpha = alpha.adjust),
                  xlim=c(0,18),ylim=c(0,1),ylab="Allele Frequency")
orig_obs_data <- hzar.extract.obsData(dors_func.dG)$frame
points(y = -3-orig_obs_data$mu, x = orig_obs_data$dist, type = "p", pch = 4, col="black")
# hzar.plot.obsData()
text(12.5,-5.1,"K-means")
# mtext(side=1,adj=1,text="Q2",line=0.8)
axis(2, at=seq(-6,3,by=1), labels = F)
axis(2, at=seq(-5,3,by=2), labels = seq(2,-6,-2), tick=F)

abline(v=gCenter(dors_func.dG), col = adjustcolor(qt.col$dors_func, alpha = alpha.adjust), 
       lty="dashed", lwd=3 )

dev.off()

