# This script does not need to be run as part of the workflow. It is provided 
# to demonstrate how the linear discriminate analysis is performed. This script 
# generates the data table used by the JYHZARadmixAnalysis.R. This script will 
# not write out the table unless you set the last ‘if’ statement to ‘TRUE’. 
# This table is provided separately as LDA_predict_ancestral_category.csv

library(hzar)
library(dplyr)
library(lattice)

PhenUpdate <- read.csv("AmNat60080_phenotype.csv", na.strings="na")
PhenKeep <- PhenUpdate %>% pull("IndNumContin")
Phen <- read.csv("HZARPhenotypeFile.csv", na.strings="na") %>%
  #filter(IndNumContin %in% PhenKeep)
  full_join(PhenUpdate %>% dplyr::select(IndNumContin,Locality,Loc_as_L,DorsalPheno,LID,Dorsal_Cluster,Dist_clust_center ),
            by=c("IndNumContin","Locality"))


locTrans <- sprintf("L%02d",1:30)
#locTrans <- sprintf("L%02d",)
names(locTrans)<-
  c("A", "B",  "C", "D", "E", "F", "G",  "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T",  "U", "V", "W", "X", "Y", "Z","AA","BB","CC","DD")
#c("A", "B",  "C", "D", "E", "F", "G",  "H", "I", "J", "K", "L", "M", "N", "O", "S", "T",  "U", "V", "W", "X", "Y", "Z","AA","BB","CC","DD")
Phen <- Phen[!as.character(Phen$Locality) %in% c("P","Q","R" ,"X","Y") , ]
Phen$Locality<- factor(locTrans[as.character(Phen$Locality)] )



siteL=c("L21","L14")
siteR=c("L23","L15")
PhenKmeans <- Phen %>% dplyr::select(Locality,Dorsal_Cluster) %>% 
  group_by(Locality) %>% summarise(dorsal.p=mean(Dorsal_Cluster==1,na.rm=T),
                                   dorsal.n=sum(!is.na(Dorsal_Cluster))) %>% filter(dorsal.n>0)

Phen_DFA_prior <- MASS::lda(DorsalPhenotype ~ DorsR+DorsG+DorsB,data = Phen)
PhenDFA_pred=predict(Phen_DFA_prior,Phen)
PhenDFA <- cbind(Phen,dorsal=PhenDFA_pred$x) %>%
  mutate(dorsal_lda=PhenDFA_pred$class)

if(F){
  print(Phen_DFA_prior)
}
Phen_train <- Phen %>% filter(Locality %in% c(siteL,siteR))%>%
  mutate(dorsal_anc=if_else(Locality %in% c(siteL),
                            "Red", "Dark"))

Phen_DFA_anc <- MASS::lda(dorsal_anc ~ DorsR+DorsG+DorsB,data = Phen_train)
PhenDFA_ancpred=predict(Phen_DFA_anc,Phen)
PhenDFA_anc <- cbind(Phen,dorsal=PhenDFA_ancpred$x) %>%
  mutate(dorsal_lda_anc=PhenDFA_ancpred$class)
with(PhenDFA_anc %>% filter(!is.na(DorsR)),table(dorsal_lda_anc, DorsalPhenotype,useNA = "ifany")) %>% 
  print.table(zero.print=".")
with(PhenDFA_anc,table(dorsal_lda_anc, DorsalPhenotype,useNA = "ifany")) %>% 
  print.table(zero.print=".")
with(PhenDFA_anc %>% filter(!is.na(DorsR)),
     table(dorsal_lda_anc, DorsalPhenotype,Dorsal_Cluster, useNA = "ifany")) %>% 
  print.table(zero.print=".")

PhenLDA <- PhenDFA_anc %>% dplyr::select(Locality,dorsal_lda_anc) %>% 
  group_by(Locality) %>% summarise(dorsal.p=mean(dorsal_lda_anc=="Dark",na.rm=T),
                                   dorsal.n=sum(!is.na(dorsal_lda_anc))) %>% filter(dorsal.n>0)

# Check observed data
pair.loc<-read.csv("locPairDistNew.csv")
loc.map<- hzar.mapSiteDist(pair.loc$Locality,pair.loc$pair.distance)
phen.kmeans <- with(PhenKmeans, 
                    hzar.doMolecularData1DPops(distance = loc.map[Locality],
                                               pObs =  dorsal.p,
                                               nEff = dorsal.n,
                                               siteID =  Locality  ))
phen.lda_anc <- with(PhenLDA, 
                    hzar.doMolecularData1DPops(distance = loc.map[Locality],
                                               pObs =  dorsal.p,
                                               nEff = dorsal.n,
                                               siteID =  Locality  ))
if(F){
pdf("KmeansDorsal.pdf",width =5,height = 5)
hzar.plot.obsData(phen.kmeans,ylab = "% Right Cluster",main="K-means Dorsal Cluster")
dev.off()

pdf("LDA_ancDorsal.pdf",width =5,height = 5)
hzar.plot.obsData(phen.lda_anc,ylab = "% Right Cluster",main="LDA Dorsal Cluster")
dev.off()
}

PhenDFA_map=PhenDFA%>%mutate(dist=loc.map[Locality])
PhenDFA_map_anc=PhenDFA_anc%>%mutate(dist=loc.map[Locality])
if(F){
  plot(dorsal.LD1 ~ dist, data=PhenDFA_map)
  plot(dorsal.LD2 ~ dist, data=PhenDFA_map)
  plot(dorsal.LD3 ~ dist, data=PhenDFA_map)
  plot(as.numeric(as.character(dorsal_lda)) ~ dist, data=PhenDFA_map)
  PhenDFA_map_anc=PhenDFA_anc%>%mutate(dist=loc.map[Locality])
  if(F){
  pdf("LDA_ancDorsal_function.pdf",width =5,height = 5)
  plot(LD1 ~ dist, data=PhenDFA_map_anc, main="DFA Ancestral Dorsal Function")
  dev.off()
  }
  plot(DorsalPhenotype ~ dist, data=PhenDFA_map)
  xyplot(dorsal_lda ~ as.factor(DorsalPhenotype), data=PhenDFA_map)
  plot(as.numeric(dorsal_lda) ~ DorsalPhenotype, data=PhenDFA_map)
  with(PhenDFA,table(dorsal_lda, DorsalPhenotype, useNA = "ifany")) %>% print.table(zero.print=".")
  PhenDFA %>% filter(dorsal_lda!=DorsalPhenotype) %>% str()
}

if(F){
  write.csv(PhenDFA_map_anc,"LDA_predict_ancestral_category.csv")

}